$NOMOD51
$INCLUDE (89C1051.MCU)
; version 21.11.98

; P3.7: Status-Anzeige
;       0  1
; P3.4: 0V/5V  auf Pin1                 RST1
; P3.5: 12V/0V auf Pin1                 RST2
; P3.3: W / E  auf Pin -7, 9,11         MODE1
; P3.7: W / E  auf Pin 8                MODE2
; P3.2: Prog / - auf Pin 6              PROG
; P3.1: XTAL1 auf Pin 5                 XTAL1

; Ersatzvariablen festlegen
        RST1  EQU P3.4
        RST2  EQU P3.5
        XTAL1 EQU P3.1
        MODE1 EQU P3.3
        MODE2 EQU P3.7
        PROG  EQU P3.2
        VIEW  EQU P3.3
; nur fr den Adaptersockel:
        MODE3 EQU P1.0
        T_IN  EQU P1.7          ; zum feststellen der prsenz des
        T_OUT EQU P1.6          ; adapters sind 2 pins verbunden

; benutzte Register:
;  R0:  berechnung des Timeout
;  R1:  steuert die Blinkfrequenz beim warten auf ein Signal
;  R2:  fr Testmodus 3 als Zhler

; 1200 Baud einstellen
        ser_Baud EQU 1200

; Startadresse des Codes:
        org 00h
        ajmp start

; hier werden die Funktionen eingebunden
        org 30h
$INCLUDE (time.inc)
$INCLUDE (uart.inc)
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; Chip initialisieren
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
start:
; Stack auf 30h hinter Registerbnke und Bit-Area setzen
        mov sp,#30h
; UART und Timer1 einrichten
        acall F_set_Baud

; angeschlossenen Controller in einen Ruhezustand bringen
        clr PROG
        setb RST1
        setb RST2
        clr XTAL1
        mov P1,#0FFh
;----------------------------------------------------------------------------
;   Hauptprogramm
;----------------------------------------------------------------------------
main:
; auf Startsignal warten
        acall wait_for_go

; was soll ich tun?
        acall F_serial_in
        mov 20h,a

; fehler?
        jnz main_w01
        ajmp sig_error
main_w01:

; lock mit adaptersockel programmieren
        push ACC
        anl ACC,#11100000b
        cjne a, #11100000b,main_w12
        pop ACC
        acall lockchip
main_w12:
        pop ACC

; lschen?
        mov c,20h.1
        jnc main_w02
        acall delchip
main_w02:

; blank check
        mov c,20h.4
        jnc main_w10
        acall blank_check
main_w10:

; schreiben?
        mov c,20h.2
        jnc main_w03
        acall progchip
main_w03:

; verify
        mov c,20h.3
        jnc main_w11
        acall verify
main_w11:

; testmodus1
        mov c,20h.5
        jnc main_w04
        acall F_serial_in
; Bit P3.0 schtzen, da sonst kein weiterer Empfang mglich
        setb ACC.0
        mov P3,a

main_w04:

; testmodus2
        mov c,20h.6
        jnc main_w05
        acall F_serial_in
        mov P1,a
main_w05:

; testmodus3
        mov c,20h.7
        jnc main_w06
        ajmp testmodus3
main_w06:

; angeschlossenen Baustein immer auf Reset legen
        setb RST1
        setb RST2

; wieder zurck zum Anfang
        ajmp main
;----------------------------------------------------------------------------
;   Funktionen
;----------------------------------------------------------------------------
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
lockchip:
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; den chip sperren, bentigt einen Adaptersockel
;  Adaptersockel feststellen
        setb T_IN
        clr T_OUT
        jnb T_IN,lockchip_ad_gef
        setb VIEW
        ajmp lockchip_stop

lockchip_ad_gef:
;  erstes LOCK-Bit programmieren
        clr PROG

; 1. Spannung anlegen
;    RST auf GND
        clr RST1
        setb RST2
;    XTAL1 auf GND
        clr XTAL1

        mov a,#1
        acall F_wait_m
;-   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   
; 2. RST auf 5V
        setb RST1
;    Prog auf H
        setb PROG

        mov a,#1
        acall F_wait_m
;-   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   
; 3. Mode whlen
        setb MODE1
        setb MODE2
        setb MODE3

        mov a,#1
        acall F_wait_m
;-   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   
; 5. Raise RST to 12V
        clr RST2

        mov a,#1
        acall F_wait_m
;-   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   
; 6. Pulse PROG once
        cpl PROG
        mov a,#11
        acall F_wait_m
        cpl PROG
;-   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   
; 10. XTAL1 auf L
        clr XTAL1
;     RST auf L
        clr RST1
        setb RST2

;  zweites LOCK-Bit programmieren
        clr PROG

; 1. Spannung anlegen
;    RST auf GND
        clr RST1
        setb RST2
;    XTAL1 auf GND
        clr XTAL1

        mov a,#1
        acall F_wait_m
;-   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   
; 2. RST auf 5V
        setb RST1
;    Prog auf H
        setb PROG

        mov a,#1
        acall F_wait_m
;-   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   
; 3. Mode whlen
        setb MODE1
        setb MODE2
        clr  MODE3

        mov a,#1
        acall F_wait_m
;-   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   
; 5. Raise RST to 12V
        clr RST2

        mov a,#1
        acall F_wait_m
;-   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   
; 6. Pulse PROG once
        cpl PROG
        mov a,#11
        acall F_wait_m
        cpl PROG
;-   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   
; 10. XTAL1 auf L
        clr XTAL1
;     RST auf L
        clr RST1
        setb RST2


        clr VIEW
lockchip_stop:
        ajmp lockchip_stop
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  verify:
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; chip auf lesen stellen        
        mov P1,#0FFh
        setb PROG
        setb RST1
        setb RST2
        clr XTAL1
        clr MODE1
        clr MODE2
        mov a,#1
        acall F_wait_m

; daten holen
go_on_verify:
        mov R0,#250
go_on_02:
        acall F_serial_in_nowait
        jnc go_on_03
; Timeout berechnen
        dec R0
        mov a,R0
        jz verify_equal
        mov a,#70
        acall F_wait_u
        ajmp go_on_02

go_on_03:

; vergleichen mit P1
        cjne A,P1,verify_not_equal

; pulse XTAL1
        cpl XTAL1
        mov a,#100
        acall F_wait_u
        cpl XTAL1

; weitermachen
        ajmp go_on_verify

verify_not_equal:
        setb VIEW
        ajmp verify_01

verify_equal:
        clr VIEW

; nach dem test anhalten, egal ob erfolgreich oder nicht!
        ajmp verify_01

;        ret

verify_01:
        setb RST1
        setb RST2
        mov P1,#0FFh
        ajmp verify_01



;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  blank_check:
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; chip auf lesen stellen
        mov P1,#0FFh
        setb PROG
        setb RST1
        setb RST2
        clr XTAL1
        clr MODE1
        clr MODE2
        mov a,#1
        acall F_wait_m

; read data byte
        mov a,P1
        inc a
; wenn nicht null, nicht leer
        jnz blank_not_blank

; test erfolgreich beendet
        ret

blank_not_blank:
        setb RST1
        setb RST2
        mov P1,#0FFh
        setb VIEW

blank_not_bl01:
        ajmp blank_not_bl01
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  testmodus3:
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
; die Zahlen 1 bis 255 einlesen
        mov R2,#1
        setb VIEW
testmodus3_0:
        acall F_serial_in
        cjne A,2,testmodus3_F
        inc R2
        mov a,R2
        jz testmodus3_G
        ajmp testmodus3_0
testmodus3_F:
        setb View
        ajmp testmodus3_E

testmodus3_G:
        clr View

testmodus3_E:
        ajmp testmodus3_E

;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; auf den Empfang von EB 98 warten
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
wait_for_go:
; auf die beiden init-bytes #EBh und #98h warten
        setb VIEW
        mov R1,#100
wait_for_go_00:

        djnz ACC,wait_for_go_02
        mov a,#0FFh
        djnz R1,wait_for_go_02
        mov R1,#100
        cpl VIEW
wait_for_go_02:

        acall F_serial_in_nowait
        jc wait_for_go_00

        cjne a,#0EBh,wait_for_go

wait_for_go_01:
        cpl VIEW
        acall F_serial_in_nowait
        jc wait_for_go_01

        cjne a,#098h,wait_for_go

        ret
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; angeschlossenen Baustein lschen
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
delchip:
        clr PROG

; 1. Spannung anlegen
;    RST auf GND
        clr RST1
        setb RST2
;    XTAL1 auf GND
        clr XTAL1

        mov a,#1
        acall F_wait_m
;-   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   
; 2. RST auf 5V
        setb RST1
;    Prog auf H
        setb PROG

        mov a,#1
        acall F_wait_m
;-   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   
; 3. Mode whlen
        setb MODE1
        clr MODE2

        mov a,#1
        acall F_wait_m
;-   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   
; 5. Raise RST to 12V
        clr RST2

        mov a,#1
        acall F_wait_m
;-   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   
; 6. Pulse PROG once
        cpl PROG
        mov a,#11
        acall F_wait_m
        cpl PROG
;-   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   
; 10. XTAL1 auf L
        clr XTAL1
;     RST auf L
        clr RST1
        setb RST2
        ret
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
; programmieren
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
progchip:
; 1. Spannung anlegen
;    RST auf GND
        clr RST1
        setb RST2
;    XTAL1 auf GND
        clr XTAL1

        mov a,#1
        acall F_wait_m
;-   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   
; 2. RST auf 5V
        setb RST1
;    Prog auf H
        setb PROG

        mov a,#1
        acall F_wait_m
;-   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   
; 3. Mode whlen
        clr MODE1
        setb MODE2

        mov a,#1
        acall F_wait_m
;-   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   
; 4. Apply data
go_on:
        mov R0,#250
go_on_2:
        acall F_serial_in_nowait
        jnc go_on_3
; Timeout berechnen
        dec R0
        mov a,R0
        jz shutdown
        mov a,#70
        acall F_wait_u
        ajmp go_on_2
;-   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   
go_on_3:

        mov P1,a
        mov a,#2
        acall F_wait_m
;-   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   
; 5. Raise RST to 12V
        clr RST2

        mov a,#1
        acall F_wait_m
;-   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   
; 6. Pulse PROG once
        cpl PROG
        mov a,#1
        acall F_wait_m
        cpl PROG
;-   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   
; 8. pulse XTAL once
        cpl XTAL1
        mov a,#1
        acall F_wait_u
        cpl XTAL1
;-   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   
; 9. repeat step 4 bis 8
        ajmp go_on

;-   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   
; 10.Power-off sequence
shutdown:
;     XTAL1 auf L
        clr XTAL1
;     RST auf L
        clr RST1
        setb RST2
        ret
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;  Fehler signalisieren
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
sig_error:
        setb VIEW       ; VIEW ist negiert
sig_error00:
        ajmp sig_error00
;----------------------------------------------------------------------------

END
