;----------------------------------------------------------------------------
;       F_serial_in:            ein Byte seriell einlesen. auf
;                               den Empfang wird gewartet
;       F_serial_in_nowait:     ein Byte seriell einlesen. auf
;                               den Empfang wird nicht gewartet
;       F_set_Baud:             die serielle bertragung vorbereiten
;                               in ser_Baud die Baudrate festlegen
;       F_serial_out:           ein Byte (ACC) auf der seriellen
;                               Schnittstelle ausgeben
;----------------------------------------------------------------------------
; 00    ein Byte von der seriellen Schnittstelle
;       einlesen und im a-register ausgeben. es wird auf das zeichen gewartet
;       die serielle Schnittstelle muss schon eingerichtet sein
F_serial_in:
; auf empfangenes Byte warten
        jnb SCON.0,F_serial_in
; wenn da, auf den ACC ausgeben das Falg lschen
        mov a,SBUF
        clr SCON.0
        ret
;----------------------------------------------------------------------------
; 01    serielle Kommunikation vorbereiten
;       baudrate muss in der Variable ser_Baud gespeichert sein
;       keine Paritt, 1 Start- 1 Stopbit, 8 Datenbits
;       benutzt wird Timer 1
; zunchst wird der Timer-Preload festgelegt
IF ser_Baud=600
        F01_z equ 152
ENDIF
IF ser_Baud=1200
        F01_z equ 204
ENDIF
IF ser_Baud=2400
        F01_z equ 230
ENDIF
IF ser_Baud=4800
        F01_z equ 243
ENDIF
IF ser_Baud=9600	
        F01_z equ 250
ENDIF
IF ser_Baud=19200	
        F01_z equ 253
ENDIF
F_set_Baud:
        mov PCON,#00h
        mov TMOD,#22h
        mov TH1, #F01_z
        mov TL1, #F01_z
        setb TCON.6
        mov SCON,#052h
        ret
;----------------------------------------------------------------------------
; 02    ein Byte von der seriellen Schnittstelle
;       einlesen und im a-register ausgeben.
;       es wird auf das zeichen NICHT gewartet
;       wenn kein Zeichen im A, dann Carry-Flag gesetzt
;       die serielle Schnittstelle muss schon eingerichtet sein
F_serial_in_nowait:
; Funktion bekannt machen

; auf empfangenes Byte warten
        jnb SCON.0,F02_weiter
; wenn da, ausgeben
        mov a,SBUF
        clr SCON.0
; Carry-flag lschen
        clr c
        ret
F02_weiter:
; Carry-flag setzen, wenn kein Byte da
        setb c
        ret
;----------------------------------------------------------------------------
;       seriell ausgeben
F_serial_out:
;  auf gesendet warten
        jnb SCON.1,F_serial_out
;  senden
        mov SBUF,a
        ret
;----------------------------------------------------------------------------
